import sys, explore
from PyQt5.QtCore import pyqtSignal, pyqtSlot, QTimer, QSignalBlocker, Qt
from PyQt5.QtGui import QPixmap, QImage
from PyQt5.QtWidgets import QLabel, QApplication, QWidget, QDesktopWidget, QCheckBox, QMessageBox, QMainWindow, QPushButton, QComboBox, QSlider, QGroupBox, QGridLayout, QBoxLayout, QHBoxLayout, QVBoxLayout, QMenu, QAction

class MainWindow(QMainWindow):
    evtCallback = pyqtSignal(int)

    @staticmethod
    def makeVLyt(lbl_1, sli_1, val_1, lbl_2, sli_2, val_2):
        hlyt_1 = QHBoxLayout()
        hlyt_1.addWidget(lbl_1)
        hlyt_1.addStretch()
        hlyt_1.addWidget(val_1)
        hlyt_2 = QHBoxLayout()
        hlyt_2.addWidget(lbl_2)
        hlyt_2.addStretch()
        hlyt_2.addWidget(val_2)
        vlyt = QVBoxLayout()
        vlyt.addLayout(hlyt_1)
        vlyt.addWidget(sli_1)
        vlyt.addSpacing(5)
        vlyt.addLayout(hlyt_2)
        vlyt.addWidget(sli_2)
        return vlyt

    def __init__(self):
        super().__init__()
        self.hcam = None
        self.timer = QTimer()
        self.imgWidth = 0
        self.imgHeight = 0
        self.pData = None
        self.res = 0
        self.temp = explore.EXPLORE_TEMP_DEF
        self.tint = explore.EXPLORE_TINT_DEF
        self.count = 0
        self.setMinimumSize(1024, 768)

        gbox_device = QGroupBox("Camera")
        self.cmb_camera = QComboBox()
        vlyt_cam = QVBoxLayout()
        vlyt_cam.addWidget(self.cmb_camera)
        gbox_device.setLayout(vlyt_cam)
        self.cmb_camera.currentIndexChanged.connect(self.onCameraChanged)

        gbox_res = QGroupBox("Resolution")
        self.cmb_res = QComboBox()
        vlyt_res = QVBoxLayout()
        vlyt_res.addWidget(self.cmb_res)
        gbox_res.setLayout(vlyt_res)
        self.cmb_res.currentIndexChanged.connect(self.onResolutionChanged)

        gbox_exp = QGroupBox("Exposure")
        self.cbox_auto = QCheckBox()
        self.cbox_auto.setEnabled(False)
        self.cbox_auto.setCheckState(Qt.CheckState.Checked)
        lbl_auto = QLabel("Auto exposure")
        hlyt_auto = QHBoxLayout()
        hlyt_auto.addWidget(self.cbox_auto)
        hlyt_auto.addSpacing(10)
        hlyt_auto.addWidget(lbl_auto)
        hlyt_auto.addStretch()
        lbl_time = QLabel("Time(us):")
        lbl_gain = QLabel("Gain(%):")
        self.lbl_expoTime = QLabel("0")
        self.lbl_expoGain = QLabel("0")
        self.slider_expoTime = QSlider(Qt.Horizontal)
        self.slider_expoGain = QSlider(Qt.Horizontal)

        self.slider_expoTime.setEnabled(False)
        self.slider_expoGain.setEnabled(False)
        glyt_exp = self.makeVLyt(lbl_time, self.slider_expoTime, self.lbl_expoTime, lbl_gain, self.slider_expoGain, self.lbl_expoGain)
        vlyt_exp = QVBoxLayout()
        vlyt_exp.addLayout(hlyt_auto)
        vlyt_exp.addLayout(glyt_exp)
        gbox_exp.setLayout(vlyt_exp)
        self.cbox_auto.stateChanged.connect(self.onAutoExpo)
        self.slider_expoTime.valueChanged.connect(self.onExpoTime)
        self.slider_expoGain.valueChanged.connect(self.onExpoGain)

        gbox_wb = QGroupBox("White balance")
        self.btn_autoWB = QPushButton("White balance")
        self.btn_autoWB.setEnabled(False)
        self.btn_autoWB.clicked.connect(self.onAutoWB)
        lbl_temp = QLabel("Temperature:")
        lbl_tint = QLabel("Tint:")
        self.lbl_temp = QLabel(str(explore.EXPLORE_TEMP_DEF))
        self.lbl_tint = QLabel(str(explore.EXPLORE_TINT_DEF))
        self.slider_temp = QSlider(Qt.Horizontal)
        self.slider_tint = QSlider(Qt.Horizontal)
        self.slider_temp.setRange(explore.EXPLORE_TEMP_MIN, explore.EXPLORE_TEMP_MAX)
        self.slider_temp.setValue(explore.EXPLORE_TEMP_DEF)
        self.slider_tint.setRange(explore.EXPLORE_TINT_MIN, explore.EXPLORE_TINT_MAX)
        self.slider_tint.setValue(explore.EXPLORE_TINT_DEF)
        self.slider_temp.setEnabled(False)
        self.slider_tint.setEnabled(False)
        glyt_wb = self.makeVLyt(lbl_temp, self.slider_temp, self.lbl_temp, lbl_tint, self.slider_tint, self.lbl_tint)
        vlyt_wb = QVBoxLayout()
        vlyt_wb.addLayout(glyt_wb)
        gbox_wb.setLayout(vlyt_wb)
        vlyt_wb.addWidget(self.btn_autoWB)
        self.slider_temp.valueChanged.connect(self.onWBTemp)
        self.slider_tint.valueChanged.connect(self.onWBTint)

        self.btn_open = QPushButton("Open")
        self.btn_open.setEnabled(False)
        self.btn_open.clicked.connect(self.onBtnOpen)
        self.btn_snap = QPushButton("Snap")
        self.btn_snap.setEnabled(False)
        self.btn_snap.clicked.connect(self.onBtnSnap)

        vlyt_ctrl = QVBoxLayout()
        vlyt_ctrl.addWidget(gbox_device)
        vlyt_ctrl.addWidget(gbox_res)
        vlyt_ctrl.addWidget(gbox_exp)
        vlyt_ctrl.addWidget(gbox_wb)
        vlyt_ctrl.addWidget(self.btn_open)
        vlyt_ctrl.addWidget(self.btn_snap)
        vlyt_ctrl.addStretch()
        wg_ctrl = QWidget()
        wg_ctrl.setLayout(vlyt_ctrl)

        self.lbl_frame = QLabel()
        self.lbl_video = QLabel()
        vlyt_show = QVBoxLayout()
        vlyt_show.addWidget(self.lbl_video, 1)
        vlyt_show.addWidget(self.lbl_frame)
        wg_show = QWidget()
        wg_show.setLayout(vlyt_show)

        grid_main = QGridLayout()
        grid_main.setColumnStretch(0, 1);
        grid_main.setColumnStretch(1, 4);
        grid_main.addWidget(wg_ctrl)
        grid_main.addWidget(wg_show)
        w_main = QWidget()
        w_main.setLayout(grid_main)
        self.setCentralWidget(w_main)

        self.arr = explore.Explorecam.EnumV2()
        if self.arr:
            self.timer.timeout.connect(self.onTimer)
            self.btn_open.setEnabled(True)
            self.evtCallback.connect(self.onevtCallback)
        for it in self.arr:
            self.cmb_camera.addItem(it.displayname)

    def onTimer(self):
        if self.hcam:
            nFrame, nTime, nTotalFrame = self.hcam.get_FrameRate()
            self.lbl_frame.setText("{}, fps = {:.1f}".format(nTotalFrame, nFrame * 1000.0 / nTime))

    def closeCamera(self):
        if self.hcam:
            self.hcam.Close()
            self.hcam = None
            self.pData = None

        self.btn_open.setText("Open")
        self.timer.stop()
        self.lbl_frame.clear()
        self.cmb_camera.setEnabled(True)
        self.cbox_auto.setCheckState(Qt.CheckState.Checked)
        self.cbox_auto.setEnabled(False)
        self.slider_expoGain.setEnabled(False)
        self.slider_expoTime.setEnabled(False)
        self.btn_autoWB.setEnabled(False)
        self.slider_temp.setEnabled(False)
        self.slider_tint.setEnabled(False)
        self.btn_snap.setEnabled(False)          

    def closeEvent(self, event):
        self.closeCamera()

    def onCameraChanged(self, index):
        self.cmb_res.clear()
        self.cur = self.arr[index]
        for i in range(0, self.cur.model.preview):
            self.cmb_res.addItem("{}*{}".format(self.cur.model.res[i].width, self.cur.model.res[i].height))

    def onResolutionChanged(self, index):
        if self.hcam:
            self.hcam.Stop() #first: stop camera

        self.res = index
        self.imgWidth = self.cur.model.res[index].width
        self.imgHeight = self.cur.model.res[index].height

        if self.hcam:
            self.startCamera() #second: restart camera

    def onAutoExpo(self, state):
        if self.hcam:
            self.hcam.put_AutoExpoEnable(1 if state else 0)
            self.slider_expoTime.setEnabled(not state)
            self.slider_expoGain.setEnabled(not state)

    def onExpoTime(self, value):
        if self.hcam:
            self.lbl_expoTime.setText(str(value))
            if not self.cbox_auto.isChecked():
                self.hcam.put_ExpoTime(value)

    def onExpoGain(self, value):
        if self.hcam:
            self.lbl_expoGain.setText(str(value))
            if not self.cbox_auto.isChecked():
                self.hcam.put_ExpoAGain(value)

    def onAutoWB(self):
        if self.hcam:
            self.hcam.AwbOnce()

    def wbCallback(nTemp, nTint, self):
        self.slider_temp.setValue(nTemp)
        self.slider_tint.setValue(nTint)

    def onWBTemp(self, value):
        if self.hcam:
            self.hcam.put_TempTint(self.temp, self.tint)
            self.lbl_temp.setText(str(value))
            self.temp = value

    def onWBTint(self, value):
        if self.hcam:
            self.hcam.put_TempTint(self.temp, self.tint)
            self.lbl_tint.setText(str(value))
            self.tint = value

    def startCamera(self):
        self.hcam.put_eSize(self.res)
        self.pData = bytes(explore.TDIBWIDTHBYTES(self.imgWidth * 24) * self.imgHeight)
        uimin, uimax, uidef = self.hcam.get_ExpTimeRange()
        self.slider_expoTime.setRange(uimin, uimax)
        self.slider_expoTime.setValue(uidef)
        usmin, usmax, usdef = self.hcam.get_ExpoAGainRange()
        self.slider_expoGain.setRange(usmin, usmax)
        self.slider_expoGain.setValue(usdef)
        self.handleExpoEvent()
        self.handleTempTintEvent()
        try:
            self.hcam.StartPullModeWithCallback(self.eventCallBack, self)
        except explore.HRESULTException:
            QMessageBox.warning(self, "Warning", "Failed to start camera")            
        else:
            self.timer.start(1000)
            self.cmb_camera.setEnabled(False)
            self.cmb_res.setEnabled(True)
            self.cbox_auto.setEnabled(True)
            self.btn_autoWB.setEnabled(True)
            self.slider_temp.setEnabled(True)
            self.slider_tint.setEnabled(True)
            self.btn_open.setText("Close")
            self.btn_snap.setEnabled(True)

    def onBtnOpen(self):
        if self.hcam:
            self.closeCamera()
        else:
            self.hcam = explore.Explorecam.Open(self.cur.id)
            if self.hcam:
                self.hcam.put_Option(explore.EXPLORE_OPTION_BYTEORDER, 0) #Qimage use RGB byte order
                self.hcam.put_AutoExpoEnable(1)
                self.startCamera()

    def onBtnSnap(self):
        if self.hcam:
            if 0 == self.cur.model.still:    # not support still image capture
                if self.pData is not None:
                    image = QImage(self.pData, self.imgWidth, self.imgHeight, QImage.Format_RGB888)
                    self.count += 1
                    image.save("pyqt{}.jpg".format(self.count))
            else:
                menu = QMenu()
                for i in range(0, self.cur.model.still):
                    action = QAction("{}*{}".format(self.cur.model.res[i].width, self.cur.model.res[i].height), self)
                    action.setData(i)
                    menu.addAction(action)
                action = menu.exec(self.mapToGlobal(self.btn_snap.pos()))
                self.hcam.Snap(action.data())

    @staticmethod
    def eventCallBack(nEvent, self):
        self.evtCallback.emit(nEvent)

    def onevtCallback(self, nEvent):
        if self.hcam:
            if explore.EXPLORE_EVENT_IMAGE == nEvent:
                self.handleImageEvent()
            elif explore.EXPLORE_EVENT_EXPOSURE == nEvent:
                self.handleExpoEvent()
            elif explore.EXPLORE_EVENT_TEMPTINT == nEvent:
                self.handleTempTintEvent()
            elif explore.EXPLORE_EVENT_STILLIMAGE == nEvent:
                self.handleStillImageEvent()

    def handleImageEvent(self):
        try:
            self.hcam.PullImageV2(self.pData, 24, None)
        except explore.HRESULTException:
            pass
        else:
            image = QImage(self.pData, self.imgWidth, self.imgHeight, QImage.Format_RGB888)
            newimage = image.scaled(self.lbl_video.width(), self.lbl_video.height(), Qt.KeepAspectRatio, Qt.FastTransformation)
            self.lbl_video.setPixmap(QPixmap.fromImage(newimage))

    def handleExpoEvent(self):
        time = self.hcam.get_ExpoTime()
        gain = self.hcam.get_ExpoAGain()
        with QSignalBlocker(self.slider_expoTime):
            self.slider_expoTime.setValue(time)
        with QSignalBlocker(self.slider_expoGain):
            self.slider_expoGain.setValue(gain)
        self.lbl_expoTime.setText(str(time))
        self.lbl_expoGain.setText(str(gain))

    def handleTempTintEvent(self):
        nTemp, nTint = self.hcam.get_TempTint()
        with QSignalBlocker(self.slider_temp):
            self.slider_temp.setValue(nTemp)
        with QSignalBlocker(self.slider_tint):
            self.slider_tint.setValue(nTint)
        self.lbl_temp.setText(str(nTemp))
        self.lbl_tint.setText(str(nTint))

    def handleStillImageEvent(self):
        info = explore.ExplorecamFrameInfoV2()
        try:
            self.hcam.PullStillImageV2(None, 24, info) # peek
        except explore.HRESULTException:
            pass
        else:
            if info.width > 0 and info.height > 0:
                buf = bytes(explore.TDIBWIDTHBYTES(info.width * 24) * info.height)
                try:
                    self.hcam.PullStillImageV2(buf, 24, info)
                except explore.HRESULTException:
                    pass
                else:
                    image = QImage(buf, info.width, info.height, QImage.Format_RGB888)
                    self.count += 1
                    image.save("pyqt{}.jpg".format(self.count))    

if __name__ == '__main__':
    app = QApplication(sys.argv)
    w = MainWindow()
    w.show()
    sys.exit(app.exec_())